/*
 * @(#)EarthTiles.java	1.1 97/03/24
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


/*
This little example contains three inner classes: a new kind of
component and two anonymous mouse listener classes.  It would
be most effective as an applet, you could run it in HotJava.
This has the advantage of eliminating the main (and the fourth 
nested class) which doesn't add much value of the example.

Note that Tile is a "lightweight" component, instances 
don't consume any underlying toolkit resources.  It's
currently necessary to repaint a lightweight component
manually after adding it to the component tree.
This line:
    add(box).repaint()
uses the add() return value to make it clear that this
repainting is a special case.
*/

     
import java.awt.*; 
import java.awt.event.*;

public class EarthTiles extends java.applet.Applet {
  float hue;
  protected Color nextColor() {
    hue = (hue + 0.06f) % 1.0f;
    return Color.getHSBColor(hue, 0.4f, 0.65f);
  }

  public EarthTiles() {
    addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
	new Tile(e.getX(), e.getY(), nextColor());
      }
    });
  }

  private final class Tile extends Component {
    Tile(int x, int y, Color color) {
      addMouseMotionListener(new MouseMotionAdapter() {
	public void mouseDragged(MouseEvent e) {
	  int bx = e.getX() + getLocation().x;
	  int by = e.getY() + getLocation().y;
	  setLocation(bx, by);
	}
      });
      setBounds(x, y, 32, 32);
      setForeground(color);
      EarthTiles.this.add(this).repaint();
    }

    public void paint(Graphics g) {
      super.paint(g);
      g.setColor(getForeground());
      Dimension d = getSize();
      g.fill3DRect(0, 0, d.width-1, d.height-1, true);
    }
  }


  public static void main(String[] args) {
    Frame f = new Frame("EarthTiles");

    // a cheat for win95:
    f.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	System.exit(0);
      }
    });

    f.add(new EarthTiles());
    f.setSize(350, 170);
    f.show();
  }
}
