/*
 * @(#)ReCode.java	1.1 97/03/24
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.io.*;

/**
 * Demonstrates interrelations between 8-bit byte streams,
 * 16-bit char streams, and strings.
 * <p>
 * Note that all constructor arguments named "*enc" are optional,
 * and default to something native to the platform.
 */
public class ReCode {
  public static void recode(InputStream src, String srcEnc,
			    OutputStream dst, String dstEnc)
			    throws IOException {
    Reader r = new InputStreamReader(src, srcEnc);
    Writer w = new OutputStreamWriter(dst, dstEnc);
    char buf[] = new char[1 << 12];
    int nr;
    while ((nr = r.read(buf)) > 0) {
      w.write(buf, 0, nr);
    }
    w.flush();
  }

  public static void recodeLines(InputStream src, String srcEnc,
			    OutputStream dst, String dstEnc)
			    throws IOException {
    BufferedReader r = new BufferedReader(new InputStreamReader(src, srcEnc));
    String line;
    while ((line = r.readLine()) != null) {
      byte bytes[] = line.getBytes(dstEnc);
      dst.write(bytes);
      dst.write('\n');
    }
    dst.flush();
  }

  public static void main(String av[]) {
    try {

      // Buffer early and often!
      // (JDK 1.1.1 handles redundant buffering without extra copying.)
      InputStream in = new BufferedInputStream(System.in);
      OutputStream out = new BufferedOutputStream(System.out);

      if (av.length == 2) {
	recode(in, av[0], out, av[1]);
      } else {
	// same op, different coding technique
	recodeLines(in, av[0], out, av[1]);
      }
    } catch (Exception ee) {
      System.err.println("Error: " + ee);
      System.err.println("Usage: java ReCode < src srcEnc > dst dstEnc");
      System.err.println("  enc = 8859_1 | UTF8 | SJIS | Unicode | ...");
    }
  }
}
